% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%% This code is used to compare the proposed handover necessity estimation 
%% (HNE) with other handover decision methods, including fixed RSS based 
%% and hysteresis based methods.
clear all

%% Network parameters initialization.
beta =3.5;
P_tx=20;
PL_ref=40;
R=150;
R_fixed=150;
R_hyst=120;
RSS_fixed=P_tx-PL_ref-10*beta*log10(R_fixed);
RSS_hyst=P_tx-PL_ref-10*beta*log10(R_hyst);
RSS_HNE=P_tx-PL_ref-10*beta*log10(R);
sigma0=4.3;
d_sample=30;
tau_i=2;
tau_o=2;
P_f=0.02;
P_u=0.04;

%% Velocities of mobile terminals from 1 km/h to 100 km/h in 2 km/h 
%% increments are simulated.
v0=[1:2:100];
for i=1:length(v0)    
    v(i)=v0(i).*5/18;
    num_failure_fixed(i) = 0;
    num_failure_hyst(i) = 0;
    num_failure_HNE(i) = 0;
    num_unnecessary_fixed(i) = 0;
    num_unnecessary_hyst(i) = 0;
    num_unnecessary_HNE(i) = 0;
    totalnum_handover_fixed(i) = 0;
    totalnum_handover_hyst(i) = 0;
    totalnum_handover_HNE_pf(i) = 0;
    totalnum_handover_HNE_pu(i) = 0;
  %% Generating 10000 trajectories with random entry and exist angles.
    for j = 1 : 10000
        theta_i(i,j)=2*pi*rand(1);
        theta_o(i,j)=2*pi*rand(1);
        theta(i,j)=theta_i(i,j)-theta_o(i,j);
        if theta(i,j) < 0
            theta(i,j) = abs(theta(i,j));
        end
        %% The length of sampling interval and number of RSS samples per
        %% second vary with the velocity of the mobile terminal.
        sample_interval(i)=0.001.*v(i);
        N(i)=floor(d_sample/sample_interval(i));
        sigma(i)=getsigma(sigma0,N(i));
        
        %% Theoretical traveling distance for the fixed RSS based method.
        d_fixed(i,j)=sqrt(2*R_fixed^2*(1-cos(theta(i,j)))); 
        l_OM_fixed(i,j) = sqrt(R_fixed^2-(d_fixed(i,j)/2).^2);
        %% Actual traveling distance for the fixed RSS based method.
        noise_fixed(i,j) = getnoise(sigma(i));
        RSS_fixed_est(i,j) = RSS_fixed+noise_fixed(i,j);
        R_fixed_est(i,j) = 10^((P_tx-PL_ref-RSS_fixed_est(i,j))/10/beta);
        d_fixed_est(i,j)=sqrt(2*R_fixed_est(i,j).^2.*(1-cos(theta(i,j))));
        l_OM_fixed_est(i,j) = sqrt(R_fixed_est(i,j).^2-(d_fixed_est(i,j)/2).^2);
        %% Calculate the number of handover failures and unnecessary
        %% handovers for the fixed RSS based method.
        if l_OM_fixed(i,j)<=R_fixed_est(i,j)
           totalnum_handover_fixed(i)=totalnum_handover_fixed(i)+1; 
           if d_fixed_est(i,j)<v(i)*tau_i 
               %% A handover failure (also an unnecessary handover) happens.
           num_failure_fixed(i)=num_failure_fixed(i)+1; 
           num_unnecessary_fixed(i)=num_unnecessary_fixed(i)+1;
           elseif d_fixed_est(i,j)<v(i)*(tau_i+tau_o) 
               %% An unnecessary handover happens.
           num_unnecessary_fixed(i)=num_unnecessary_fixed(i)+1;
           end
        end
        
        %% Theoretical traveling distance for the hysteresis based method.
        d_hyst(i,j)=sqrt(2*R_hyst^2*(1-cos(theta(i,j)))); 
        l_OM_hyst(i,j) = sqrt(R_hyst^2-(d_hyst(i,j)/2).^2);
        %% Actual traveling distance for the hysteresis based method.
        noise_hyst(i,j) = getnoise(sigma(i));
        RSS_hyst_est(i,j) = RSS_hyst+noise_hyst(i,j);
        R_hyst_est(i,j) = 10^((P_tx-PL_ref-RSS_hyst_est(i,j))/10/beta);
        d_hyst_est(i,j)=sqrt(2*R_hyst_est(i,j).^2.*(1-cos(theta(i,j))));
        l_OM_hyst_est(i,j) = sqrt(R_hyst_est(i,j).^2-(d_hyst_est(i,j)/2).^2);
        %% Calculate the number of handover failures and unnecessary
        %% handovers for the hysteresis method.
        if l_OM_hyst(i,j)<=R_hyst_est(i,j)
           totalnum_handover_hyst(i)=totalnum_handover_hyst(i)+1;
           if d_hyst_est(i,j)<v(i)*tau_i 
               %% A handover failure (also an unnecessary handover) happens.
           num_failure_hyst(i)=num_failure_hyst(i)+1; 
           num_unnecessary_hyst(i)=num_unnecessary_hyst(i)+1;         
           elseif d_hyst_est(i,j)<v(i)*(tau_i+tau_o) 
               %% An unnecessary handover happens.
           num_unnecessary_hyst(i)=num_unnecessary_hyst(i)+1;
           end
        end
        
        %% Theoretical traveling distance for HNE
        d_HNE(i,j)=sqrt(2*R^2*(1-cos(theta(i,j)))); 
        l_OM_HNE(i,j) = sqrt(R^2-(d_HNE(i,j)/2).^2);
        %% Actual traveling distance for HNE
        noise_HNE(i,j) = getnoise(sigma(i));
        RSS_HNE_est(i,j) =RSS_HNE+noise_HNE(i,j);
        R_HNE_est(i,j) = 10^((P_tx-PL_ref-RSS_HNE_est(i,j))/10/beta);
        d_HNE_est(i,j)=sqrt(2*R_HNE_est(i,j).^2.*(1-cos(theta(i,j))));
        l_OM_HNE_est(i,j) = sqrt(R_HNE_est(i,j).^2-(d_HNE_est(i,j)/2).^2);
        %% Calculate the number of handover failures and unnecessary
        %% handovers for HNE
        if l_OM_HNE(i,j)<=R_HNE_est(i,j) 
            %% If not, no handover happens.
           D1=2*R*sin(asin(v(i)*tau_i/2/R)-pi/2*P_f); 
           %% Distance threshold for minimizing handover failures.
           D2=2*R*sin(asin(v(i)*(tau_i+tau_o)/2/R)-pi/2*P_u); 
           %% Distance threshold for minimizing unnecessary handovers.
           if d_HNE(i,j)>=D1
              totalnum_handover_HNE_pf(i)=totalnum_handover_HNE_pf(i)+1;
              if d_HNE_est(i,j)<v(i)*tau_i
                 num_failure_HNE(i)=num_failure_HNE(i)+1; 
                 %% Handover failure happens.
              end
           end
           if d_HNE(i,j)>=D2
              totalnum_handover_HNE_pu(i)=totalnum_handover_HNE_pu(i)+1;
              if d_HNE_est(i,j)<v(i)*(tau_i+tau_o)
                 num_unnecessary_HNE(i)=num_unnecessary_HNE(i)+1; 
                 %% Unnecessary handover happens.
              end
           end
        end        
    end    
end

%% Plot the figure for number of handover failures.  
plot(v0,num_failure_fixed,'-.b','LineWidth',2);
hold on
plot(v0,num_failure_hyst,'--g','LineWidth',2);
hold on
plot(v0,num_failure_HNE,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE for P_f=0.02');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Number of Handover Failures')

%% Plot the figure for number of unnecessary handovers.  
figure
plot(v0,num_unnecessary_fixed,'-.b','LineWidth',2);
hold on
plot(v0,num_unnecessary_hyst,'--g','LineWidth',2);
hold on
plot(v0,num_unnecessary_HNE,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE for P_u=0.04');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Number of Unnecessary Handovers')
 
%% Plot the figure for total number of handovers.  
figure
plot(v0,totalnum_handover_fixed,'-.b','LineWidth',2);
hold on
plot(v0,totalnum_handover_hyst,'--g','LineWidth',2);
hold on
plot(v0,totalnum_handover_HNE_pf,'-y','LineWidth',2);
hold on
plot(v0,totalnum_handover_HNE_pu,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE for P_f=0.02','HNE for P_u=0.04');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Total Number of Handovers') 

%% Plot the figure for ratio of number of handover failures to total number
%% of handovers.  
figure
plot(v0,num_failure_fixed./totalnum_handover_fixed,'-.b','LineWidth',2);
hold on
plot(v0,num_failure_hyst./totalnum_handover_hyst,'--g','LineWidth',2);
hold on
plot(v0,num_failure_HNE./totalnum_handover_HNE_pf,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE for P_f=0.02');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Ratio of Number of Handover Failures to Total Number of Handovers')    
    
%% Plot the figure for ratio of number of unnecessary handovers to total 
%% number of handovers.
figure
plot(v0,num_unnecessary_fixed./totalnum_handover_fixed,'-.b','LineWidth',2);
hold on
plot(v0,num_unnecessary_hyst./totalnum_handover_hyst,'--g','LineWidth',2);
hold on
plot(v0,num_unnecessary_HNE./totalnum_handover_HNE_pu,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE for P_u=0.04');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Ratio of Number of Unnecessary Handovers to Total Number of Handovers')
