% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%% This code is used to compare the proposed handover triggering condition 
%% estimation (HTCE) with other handover triggering methods, including 
%% fixed RSS based and hysteresis based methods.
clear all

%% Network parameters initialization.
R=100;Et=500;beta =3.5;
R_fixed=90;
RSS_fixed=10*log10(Et)-10*beta*log10(R_fixed);
R_hyst=80;
RSS_hyst=10*log10(Et)-10*beta*log10(R_hyst);
sigma0=4.3;
N=20;
sigma=getsigma(sigma0,N);
tau_i=1;
tau_o=1;
beta=3.5;
P_b=0;
v0=[1:5:150];
er=0.1;
m=0;
%-----------------------------------------------

for i=1:length(v0)    
    v(i)=v0(i).*5/18;
    num_breakdown_fixed(i) = 0;
    num_breakdown_hyst(i) = 0;
    num_breakdown_HTCE(i) = 0;
    esterror_ave(i) = 0;
    esterror_sum(i) = 0;
    
    for j = 1 : 1000 %Generating 1000 trajectories with random entry and exist angles
        theta_i(i,j)=2*pi*rand(1);
        theta_o(i,j)=2*pi*rand(1);
        theta(i,j)=theta_i(i,j)-theta_o(i,j);
        if theta(i,j) < 0
            theta(i,j) = abs(theta(i,j));
        end
        d(i,j)=sqrt(2*R^2*(1-cos(theta(i,j)))); 
        l_OM(i,j) = sqrt(R^2-(d(i,j)/2).^2);
        RSS_M(i,j)=10*log10(Et)-10*beta*log10(l_OM(i,j));
        d_R(i,j)=1/2*d(i,j);
        
        %Theoretical traveling distance for the fixed RSS threshold method
        if l_OM(i,j)<R_fixed %Condition C1
            d_r_fixed(i,j)=sqrt(R_fixed^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_fixed(i,j)=d_R(i,j)-d_r_fixed(i,j);
        else %Condition C2
            d_b_fixed(i,j)=d_R(i,j);
        end
        %Actual traveling distance for the fixed RSS threshold method
        noise1_fixed(i,j) = getnoise(sigma);
        RSS_M_fixed_est(i,j) = RSS_M(i,j)+noise1_fixed(i,j);
        l_OM_fixed_est(i,j) = 10^((10*log10(Et)-RSS_M_fixed_est(i,j))/10/beta);
        if l_OM_fixed_est(i,j)<R_fixed %Condition C1
            noise2_fixed(i,j) = getnoise(sigma);
            RSS_fixed_est(i,j) = RSS_fixed+noise2_fixed(i,j);
            R_fixed_est(i,j) = 10^((10*log10(Et)-RSS_fixed_est(i,j))/10/beta);
            if R_fixed_est(i,j)>=R
                num_breakdown_fixed(i) = num_breakdown_fixed(i)+1;
            elseif R_fixed_est(i,j)<=l_OM(i,j)
                if d_R(i,j)<v(i)*tau_o
                    num_breakdown_fixed(i) = num_breakdown_fixed(i)+1;
                end                
            else
            d_r_fixed_est(i,j)=sqrt(R_fixed_est(i,j)^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_fixed_est(i,j)=d_R(i,j)-d_r_fixed_est(i,j);
            if d_b_fixed_est(i,j)<=v(i)*tau_o
                num_breakdown_fixed(i) = num_breakdown_fixed(i)+1;
            end
            end
        else %Condition C2
            if l_OM_fixed_est(i,j)>=R
                num_breakdown_fixed(i) = num_breakdown_fixed(i)+1;
            else
            d_R_fixed_est(i,j) = sqrt(R^2-l_OM_fixed_est(i,j)^2);
            d_b_fixed_est(i,j) = d_R_fixed_est(i,j);
            if d_b_fixed_est(i,j)<=v(i)*tau_o
                num_breakdown_fixed(i) = num_breakdown_fixed(i)+1;
            end
            end
        end
            
               
        %Theoretical traveling distance for the hysteresis method
        if l_OM(i,j)<R_hyst %Condition C1
            d_r_hyst(i,j)=sqrt(R_hyst^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_hyst(i,j)=d_R(i,j)-d_r_hyst(i,j);
        else %Condition C2
            d_b_hyst(i,j)=d_R(i,j);
        end
        %Actual traveling distance for the fixed RSS threshold method
        noise1_hyst(i,j) = getnoise(sigma);
        RSS_M_hyst_est(i,j) = RSS_M(i,j)+noise1_hyst(i,j);
        l_OM_hyst_est(i,j) = 10^((10*log10(Et)-RSS_M_hyst_est(i,j))/10/beta);
        if l_OM_hyst_est(i,j)<R_hyst %Condition C1
            noise2_hyst(i,j) = getnoise(sigma);
            RSS_hyst_est(i,j) = RSS_hyst+noise2_hyst(i,j);
            R_hyst_est(i,j) = 10^((10*log10(Et)-RSS_hyst_est(i,j))/10/beta);
            if R_hyst_est(i,j)>=R
                num_breakdown_hyst(i) = num_breakdown_hyst(i)+1;
            elseif R_hyst_est(i,j)<=l_OM(i,j)
                if d_R(i,j)<v(i)*tau_o
                    num_breakdown_hyst(i) = num_breakdown_hyst(i)+1;
                end  
            else 
            d_r_hyst_est(i,j)=sqrt(R_hyst_est(i,j)^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_hyst_est(i,j)=d_R(i,j)-d_r_hyst_est(i,j);
            if d_b_hyst_est(i,j)<=v(i)*tau_o
                num_breakdown_hyst(i) = num_breakdown_hyst(i)+1;
            end
            end
        else %Condition C2
            if l_OM_hyst_est(i,j)>=R
                num_breakdown_hyst(i) = num_breakdown_hyst(i)+1;
            else
            d_R_hyst_est(i,j) = sqrt(R^2-l_OM_hyst_est(i,j)^2);
            d_b_hyst_est(i,j) = d_R_hyst_est(i,j);
            if d_b_hyst_est(i,j)<=v(i)*tau_o
                num_breakdown_hyst(i) = num_breakdown_hyst(i)+1;
            end
            end
        end
        
        %Theoretical traveling distance for HTCE
        r(i,j) = sqrt(v(i)^2*tau_o^2+R^2-v(i)*tau_o*R*sqrt(2*(1-cos(pi-pi*P_b))))*(1-er);
        RSS_HTCE(i,j) = 10*log10(Et)-10*beta*log10(r(i,j));
        if l_OM(i,j)<r(i,j) %Condition C1
            d_r_HTCE(i,j)=sqrt(r(i,j)^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_HTCE(i,j)=d_R(i,j)-d_r_HTCE(i,j);
        else %Condition C2
            d_b_HTCE(i,j)=d_R(i,j);
        end
        %Actual traveling distance for HTCE
        noise1_HTCE(i,j) = getnoise(sigma);
        RSS_M_HTCE_est(i,j) = RSS_M(i,j)+noise1_HTCE(i,j);
        l_OM_HTCE_est(i,j) = 10^((10*log10(Et)-RSS_M_HTCE_est(i,j))/10/beta);
        if l_OM_HTCE_est(i,j)<R
        d_R_HTCE_est(i,j) = sqrt(R^2-l_OM_HTCE_est(i,j)^2);
        d_predict_est(i,j) = d_R_HTCE_est(i,j)-d_R(i,j)*(1-er)+v(i)*tau_o;
        else
            d_R_HTCE_est(i,j)=0;
            d_predict_est(i,j)=0;
        end
        if l_OM_HTCE_est(i,j)<r(i,j) %Condition C1
            noise2_HTCE(i,j) = getnoise(sigma);
            RSS_HTCE_est(i,j) = RSS_HTCE(i,j)+noise2_HTCE(i,j);
            r_est(i,j) = 10^((10*log10(Et)-RSS_HTCE_est(i,j))/10/beta);
            if r_est(i,j)>=R
                d_handover(i,j) = d_predict_est(i,j);
                d_b_HTCE_est(i,j)=0;
            elseif r_est(i,j)<=l_OM(i,j)
                d_b_HTCE_est(i,j)=d_R(i,j);
                if d_predict_est(i,j)>d_R(i,j)
                    d_handover(i,j) = d_predict_est(i,j);
                else d_handover(i,j) = d_R(i,j);
                end  
            else
            d_r_HTCE_est(i,j)=sqrt(r_est(i,j)^2-1/2*R^2*(1+cos(theta(i,j))));
            d_b_HTCE_est(i,j)=d_R(i,j)-d_r_HTCE_est(i,j);
            if d_predict_est(i,j)>d_b_HTCE_est(i,j)
                d_handover(i,j) = d_predict_est(i,j);
            else d_handover(i,j) = d_b_HTCE_est(i,j);
            end
            end
        else %Condition C2
            if l_OM_HTCE_est(i,j)>=R
                d_handover(i,j) = 0;
                d_b_HTCE_est(i,j)=0;
            else
            d_b_HTCE_est(i,j) = d_R_HTCE_est(i,j);
            if d_predict_est(i,j)>d_b_HTCE_est(i,j)
                d_handover(i,j) = d_predict_est(i,j);
            else d_handover(i,j) = d_b_HTCE_est(i,j);
            end
            end
        end
        if d_handover(i,j)<v(i)*tau_o
        num_breakdown_HTCE(i) = num_breakdown_HTCE(i)+1;
        end
        esterror(i,j) = abs(d_b_HTCE_est(i,j)-d_b_HTCE(i,j))/d_b_HTCE(i,j);
        esterror_sum(i) = esterror_sum(i)+esterror(i,j); 
        m = m+1;
        
    end
    esterror_ave(i) = esterror_sum(i)/m;
end

figure
plot(v0,num_breakdown_fixed/5000,'-.b','LineWidth',2);
hold on
plot(v0,num_breakdown_hyst/5000,'--g','LineWidth',2);
hold on
plot(v0,num_breakdown_HTCE/5000,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HTCE for P_b=0');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Number of Connection Breakdowns')

figure
plot(v0,esterror_ave*100,'-.b','LineWidth',2);
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Average Estimation Error Ratio (%)')


% figure
% plot(v0,num_failure_fixed./totalnum_handover_fixed,'-.b','LineWidth',2);
% hold on
% plot(v0,num_failure_hyst./totalnum_handover_hyst,'--g','LineWidth',2);
% hold on
% plot(v0,num_failure_HNE./totalnum_handover_HNE_pf,'-r','LineWidth',2);
% hold on
% legend('Fixed RSS based','Hysteresis based','HNE for P_f=0.02');
% grid  ('on')
% xlabel('Velocity (km/h)')
% ylabel('Ratio of Number of Handover Failures to Total Number of Handovers')    
    
